package samples.compose;

import xcat.framework.ccacore.Services;
import xcat.framework.ccacore.ComponentID;
import xcat.framework.ccacore.ServicesImpl;
import xcat.framework.creation.CreationService;
import xcat.framework.connection.ConnectionService;
import xcat.framework.util.SoapConnectionClient;
import xcat.framework.util.EnvObj;
import soaprmi.util.Logger;
import soaprmi.util.Level;
import java.io.PrintStream;
import java.io.FileOutputStream;
import java.io.FileNotFoundException;

/**
 * @version $Revision: 1.9 $  $Author: mgovinda $ $Date: 2001/05/10 16:57:01 $ (GMT) 
 * @author Madhusudhan Govindaraju [mailto:mgovinda@extreme.indiana.edu]
 */

/**
 * This compositionEngine is used to create and connect
 * the StringGenerator and PrinterComponents
 **/
public class Compose {

  public static final String RELEASE = "$Revision: 1.9 $ by $Author: mgovinda $ on $Date: 2001/05/10 16:57:01 $ (GMT)";
  private static Logger logger = Logger.getLogger();
  private boolean stopWaiting = false;
  
  public Compose() {
    // the code for the logger is commented out so that the output
    // can be seen on the console. To dump the output on to a file
    // uncomment the code in this constructor
    //    try {
    //       logger.setSink(new PrintStream(new FileOutputStream("compose.out", true)));
    //      } catch (FileNotFoundException fnfe) {
    //        fnfe.printStackTrace();
    //      }
  }

  public void runit() {
    try {
      // create the execution env for the printer component
      EnvObj envPrinterComponent = new EnvObj();
      envPrinterComponent.put("creation-proto","ssh");
      envPrinterComponent.put("creation-mech-path","/usr/local/bin/ssh");
      envPrinterComponent.put("exec-host","exodus.extreme.indiana.edu");
      envPrinterComponent.put("exec-path", "/u/mgovinda/Projects/xcat/src/java/scripts/JavaComponentScript.sh");
      envPrinterComponent.put("exec-name", "printerComponent");
      envPrinterComponent.put("exec-fqn", "samples.printer.PrinterComponent");

      // create the execution env for the stringGenerator component
      EnvObj envStringComponent = new EnvObj();
      envStringComponent.put("creation-proto","ssh");
      envStringComponent.put("creation-mech-path","/usr/local/bin/ssh");
      envStringComponent.put("exec-host","exodus.extreme.indiana.edu");
      envStringComponent.put("exec-path", "/u/mgovinda/Projects/xcat/src/java/scripts/JavaComponentScript.sh");
      envStringComponent.put("exec-name", "stringGeneratorComponent");
      envStringComponent.put("exec-fqn", "samples.stringGenerator.StringGeneratorComponent");

      // get the services object for this component.
      // Compose.java itself can be a component,
      // however, for that it has to implement the Component interface
      Services core = new ServicesImpl();
      ComponentID compositionEngineCompID = core.getComponentID();

      // the creation service port is available to 
      // every component by default
      CreationService creationService = (CreationService)core.getPort("CreationService");
      logger.finer("calling creation Service for printerGenCompID");
      // create the printer component
      ComponentID printerCompID = creationService.createInstance(envPrinterComponent);
      logger.finer("printerGenCompID created");

      logger.finer("calling creation Service for stringGenCompID");
      // create the string generator component
      ComponentID stringGenCompID = creationService.createInstance(envStringComponent);
      logger.finer("stringGenCompID created");
      
      // the connection service port is available to 
      // every component by default
      ConnectionService connectionService = 
	(ConnectionService)core.getPort("ConnectionService");
      logger.finer("calling connectionService connect");

      // Connect the two components 
      connectionService.connect(stringGenCompID, "stringUsesPort", printerCompID, "stringProvidesPort");
      logger.finer("connection Service connect succeeded ");
      
    }  catch(Exception e) {
      logger.severe("run exception", e);
    }
  }
  
  public static void main(String[] args){
    System.out.println(RELEASE);
    Compose compose = new Compose();
    compose.runit();
    System.out.println("\n Compose: End of test \n");
  }
}
