package samples.composeObject;

import xcat.framework.ccacore.Services;
import xcat.framework.ccacore.ComponentID;
import xcat.framework.ccacore.ServicesImpl;
import xcat.framework.creation.CreationService;
import xcat.framework.connection.ConnectionService;
import xcat.framework.util.SoapConnectionClient;
import xcat.framework.util.EnvObj;
import soaprmi.util.Logger;
import soaprmi.util.Level;
import java.io.PrintStream;
import java.io.FileOutputStream;
import java.io.FileNotFoundException;

/**
 * @version $Revision: 1.2 $  $Author: mgovinda $ $Date: 2001/05/10 16:57:30 $ (GMT) 
 * @author Rachana Ananthakrishnan [mailto:ranantha@extreme.indiana.edu]
 */

/**
 * This compositionEngine is used to create and connect
 * the TestObjectGenerator and TestObjectPrinter components
 **/
public class ComposeObject {

  public static final String RELEASE = "$Revision: 1.2 $ by $Author: mgovinda $ on $Date: 2001/05/10 16:57:30 $ (GMT)";
  private static Logger logger = Logger.getLogger();
  private boolean stopWaiting = false;
  
  public ComposeObject() {
    // the code for the logger is commented out so that the output
    // can be seen on the console. To dump the output on to a file
    // uncomment the code in this constructor

    //      try {
    //        logger.setSink(new PrintStream(new FileOutputStream("composeObject.out", true)));
    //       } catch (FileNotFoundException fnfe) {
    //         fnfe.printStackTrace();
    //       }
  }

  public void runit() {
    try {
      // create the execution env for the testObjectPrinter component
      EnvObj envPrinterComponent = new EnvObj();
      envPrinterComponent.put("creation-proto","ssh");
      envPrinterComponent.put("creation-mech-path","/usr/local/bin/ssh");
      envPrinterComponent.put("exec-host","exodus.extreme.indiana.edu");
      envPrinterComponent.put("exec-path", "/u/mgovinda/Projects/xcat/src/java/scripts/JavaComponentScript.sh");
      envPrinterComponent.put("exec-name", "testObjectPrinter");
      envPrinterComponent.put("exec-fqn", "samples.testObjectPrinter.TestObjectPrinterComponent");
      // create the execution env for the testObjeceGenerator component
      EnvObj envObjectComponent = new EnvObj();
      envObjectComponent.put("creation-proto","rsh");
      envObjectComponent.put("creation-mech-path","/usr/bin/rsh");
      envObjectComponent.put("exec-host","exodus.extreme.indiana.edu");
      envObjectComponent.put("exec-path", "/u/mgovinda/Projects/xcat/src/java/scripts/JavaComponentScript.sh");
      envObjectComponent.put("exec-name", "testObjectGenerator");
      envObjectComponent.put("exec-fqn", "samples.testObjectGenerator.TestObjectGeneratorComponent");

      // get the services object for this component.
      // ComposeObject.java itself can be a component,
      // however, for that it has to implement the Component interface
      Services core = new ServicesImpl();
      ComponentID compositionEngineCompID = core.getComponentID();

      // the creation service port is available to 
      // every component by default
      CreationService creationService = (CreationService)core.getPort("CreationService");

      logger.finer("calling creation Service for printerGenCompID");
      // create the testObjectPrinter component
      ComponentID printerCompID = creationService.createInstance(envPrinterComponent);
      logger.finer("printerGenCompID created");

      logger.finer("calling creation Service for objectGenCompID");
      // create the testObjectGenerator component
      ComponentID objectGenCompID = creationService.createInstance(envObjectComponent);
      logger.finer("objectGenCompID created");
      
       // the connection service port is available to 
      // every component by default
      ConnectionService connectionService = 
	(ConnectionService)core.getPort("ConnectionService");
      logger.finer("calling connectionService connect");
     // Connect the two components 
      connectionService.connect(objectGenCompID, "testObjectUsesPort", printerCompID, "testObjectProvidesPort");
      logger.finer("connection Service connect succeeded ");
      
    }  catch(Exception e) {
      logger.severe("run exception", e);
    }
  }
  
  public static void main(String[] args){
    System.out.println(RELEASE);
    ComposeObject composeObject = new ComposeObject();
    composeObject.runit();
    System.out.println("\n ComposeObject: End of test \n");
  }
}



