package samples.printer;

import xcat.framework.ccacore.Services;
import xcat.framework.ccacore.Component;
import xcat.framework.ccacore.PortInfoImpl;
import xcat.framework.ccacore.ProvidesPortMapping;
import samples.idl.string.ProvidesString;

/**
 * @version $Revision: 1.5 $  $Author: mgovinda $ $Date: 2001/05/08 20:05:26 $ (GMT) 
 * @author Madhusudhan Govindaraju [mailto:mgovinda@extreme.indiana.edu]
 */

/**
 * The printer component prints whatever strings it
 * receives on it provides port.
 */
public class PrinterComponent implements Component {
  private Services printerCore = null;
  private ProvidesString printMe = null;

  /**
   * An empty constructor for the component is required
   * as reflection is used to instantiated an instance of
   * this class using this constructor, and then the
   * setServices methods is called on it. This is
   * done in SoapComponentInstantiator.java
   */
  public PrinterComponent() {
  }

  /**
   * @param services is an instance of ServicesImpl
   * this method is part of the Component interface
   */
  public void setServices(Services cc) {
    try {
      printerCore = cc;
      // The provides port for string is
      // implemented by the class PrintMeImpl.java

      // This mapping is used to created the dynamic proxy 
      // to the provides port. This mapping should be added 
      // before the addProvidesPort(...) call is made
      ProvidesPortMapping.addMapping("stringProvidesPort",
				     "http://www.extreme.indiana.edu/xcat/samples.wsdl#string",
				     new Class[] {ProvidesString.class}
				     );
      
      // an instance of the provides port needs to be 
      // given as part of the addProvidesPort(...) API.
      // the port method will be invoked on this instance
      printMe = new PrintMeImpl();

      // standard CCA call to add a provides port to the framework
      printerCore.addProvidesPort
	(printMe, 
	 new PortInfoImpl(
			  "stringProvidesPort", 
			  "http://www.extreme.indiana.edu/xcat/samples.wsdl#string"
			  )
	   );
    } catch (Exception e) {
      e.printStackTrace();
    }
  }
}
