package samples.stringGenerator;

import xcat.framework.ccacore.Services;
import xcat.framework.ccacore.Component;
import xcat.framework.ccacore.PortInfoImpl;
import xcat.framework.ccacore.PortInfo;
import xcat.framework.ccacore.UsesPortMapping;
import samples.idl.string.UsesString;
import soaprmi.util.Logger;
import soaprmi.util.Level;

/**
 * @version $Revision: 1.8 $  $Author: mgovinda $ $Date: 2001/05/08 23:43:05 $ (GMT) 
 * @author Madhusudhan Govindaraju [mailto:mgovinda@extreme.indiana.edu]
 */

/**
 * The stringGenerator component generates two strings and
 * writes it to its uses port
 */
public class StringGeneratorComponent implements Component  {
  private Services stringGeneratorCore;
  private static Logger logger = Logger.getLogger();
  
  /**
   * An empty constructor for the component is required
   * as reflection is used to instantiated an instance of
   * this class using this constructor, and then the
   * setServices methods is called on it. This is
   * done in SoapComponentInstantiator.java
   */
  public StringGeneratorComponent() {
  }
  
  /**
   * @param services is an instance of ServicesImpl
   * this method is part of the Component interface
   */
  public void setServices(Services services){
    stringGeneratorCore = services;
    try {
      // This mapping is used to created the dynamic proxy 
      // to the uses port. This mapping should be added 
      // before the registerUsesPort(...) call is made
      UsesPortMapping.addMapping("stringUsesPort",
				 "http://www.extreme.indiana.edu/xcat/samples.wsdl#string",
				 new Class[] {UsesString.class}
				 );

      // standard cca call to register uses port
      stringGeneratorCore.registerUsesPort(new PortInfoImpl("stringUsesPort", "http://www.extreme.indiana.edu/xcat/samples.wsdl#string")); 
    } catch(Exception e) {
      e.printStackTrace();
      logger.severe("StringGeneratorComponent: unable to register: ", e);
    }
    // start an anonymous thread to wait
    (new Thread() {
	public void run() {
	  doStuff();
	}
      }).start();
  }

  /**
   * wait for 5 seconds before executing
   * this is a substitute till the "go" port
   * is implemented.
   */
  public void doStuff() {
    try {
      synchronized(this) {
	wait(5000);
	startExecuting();
      }
    } catch(InterruptedException ex) {
    }
  }

  /**
   * This method represents whatever computation 
   * the component wants to do
   */
  public void startExecuting() {
    try {
      logger.finer("StringGenerator Component:: execute");
      // the mapping in setServices used UsesString.class
      // and hence UsesString is used here.
      UsesString output;
      // "stringUsesPort" is the same as the name of the
      // port added in registerUsesPort
      output = (UsesString)stringGeneratorCore.getPort("stringUsesPort");
      logger.fine("retrieved uses port for stringUsesPort " + output.getClass());
      // now invoke methods on the usesport
      output.sendString("[StringGenerator] xCAT is better than XCAT ");
      output.sendString("[StringGenerator] If you didn't receive the previous message ignore it, hmm ... is that possible ? ");
      logger.fine("done invoking methods on the uses port");
      stringGeneratorCore.releasePort("stringUsesPort");
      logger.fine("Component:: startExecuting done ");
    } catch (Exception e) {
      e.printStackTrace();
    }
  }
}

