package samples.testObjectPrinter;

import xcat.framework.ccacore.Services;
import xcat.framework.ccacore.Component;
import xcat.framework.ccacore.PortInfoImpl;
import xcat.framework.ccacore.ProvidesPortMapping;
import samples.idl.testObject.ProvidesTestObject;

/**
 * @version $Revision: 1.1.1.1 $  $Author: mgovinda $ $Date: 2001/05/08 20:24:19 $ (GMT) 
 * @author Rachana Ananthakrishnan [mailto:ranantha@extreme.indiana.edu]
 */

/**
 * The testObjectPrinter component prints whatever is
 * invoked on its provides port and returns the same
 * value back 
 */
public class TestObjectPrinterComponent implements Component {
  private Services objectPrinterCore = null;
  private ProvidesTestObject printMe = null;

  /**
   * An empty constructor for the component is required
   * as reflection is used to instantiated an instance of
   * this class using this constructor, and then the
   * setServices methods is called on it. This is
   * done in SoapComponentInstantiator.java
   */
  public TestObjectPrinterComponent() {
  }

  public void setServices(Services cc) {
    try {
      objectPrinterCore = cc;
      // The provides port for testObject is
      // implemented by the class PrintMeImpl.java

      // This mapping is used to created the dynamic proxy 
      // to the provides port. This mapping should be added 
      // before the addProvidesPort(...) call is made
      ProvidesPortMapping.addMapping("testObjectProvidesPort",
				     "http://www.extreme.indiana.edu/xcat/samples/wsdl#testObject",
				     new Class[] {ProvidesTestObject.class}
				     );
      // an instance of the provides port needs to be 
      // given as part of the addProvidesPort(...) API.
      // the port method will be invoked on this instance
      printMe = new PrintMeImpl();

      // standard CCA call to add a provides port to the framework
      objectPrinterCore.addProvidesPort
	(printMe, 
	 new PortInfoImpl(
			  "testObjectProvidesPort", 
			  "http://www.extreme.indiana.edu/xcat/samples/wsdl#testObject"
			  )
	   );
    } catch (Exception e) {
      e.printStackTrace();
    }
  }
}



