
/**
 * @version $Revision: 1.4 $ $Author: mgovinda $ $Date: 2001/05/08 23:43:06 $ (GMT) 
 * @author Madhusudhan Govindaraju [mailto:mgovinda@extreme.indiana.edu]
 **/

package xcat.framework.ccacore;
import xcat.framework.exceptions.XCATException;

public interface Services {
  /** 
   * Creates a PortInfo to be used in subsequent calls to describe a Port. 
   */
  public PortInfo createPortInfo(String name, 
				 String type, 
				 String[] properties);
  
  /** 
   * Ask for a previously registered Port.
   * will return a Port or throw an exception. 
   */
  public Port getPort(String name) throws XCATException;
  
  /**
   * Ask for a previously registered Port and return 
   * that Port if it is available or return null otherwise. 
   * Error in the spec ? The spec should specify that an exception 
   * should be thrown in case of an error instead of 
   * returning null.
   */
  public Port getPortNonblocking(String name);
  
  /** 
   * Notifies the framework that this component is 
   * finished with this Port. releasePort() method calls exactly 
   * match getPort() method calls.  After releasePort() is invoked 
   * all references to the released Port become invalid. 
   * XCAT specifics: a exception is thrown if a port with
   * the given name is not found.
   */
  public void releasePort(String name) throws XCATException;
  
  /** 
   * Register a request for a Port that will 
   * be retrieved subsequently with a call to getPort(). 
   */
  public void registerUsesPort(PortInfo nameAndType) 
    throws XCATException;
  
  /** 
   * Notify the framework that a Port, previously 
   * registered by this component, is no longer desired. 
   * XCAT specifics: throw an exception if a port with
   * the specified name was not registered earlier.
   */
  public void unregisterUsesPort(String name) throws XCATException;
  
  /** 
   * Exports a Port implemented by this component to the framework.  
   * This Port is now available for the framework to connect to 
   * other components. 
   * XCAT specifics: an exception is thrown if no port exists with
   * the type that is specified with the portInfo
   */
  public void addProvidesPort(Port inPort, PortInfo portInfo) 
    throws XCATException;
  
  /** 
   * Notifies the framework that a previously exported Port 
   * is no longer available for use. 
  */
  public void removeProvidesPort(String name) 
    throws XCATException;

  /** 
   * Get a reference to the component to which this Services 
   * object belongs. 
   */
  public ComponentID  getComponentID() throws XCATException;
}
  
