package xcat.framework.ccacore;

import xcat.framework.creation.CreationService;
import xcat.framework.connection.ConnectionService;
import xcat.framework.exceptions.XCATException;
import soaprmi.Remote;
import soaprmi.util.Logger;
import soaprmi.util.Level;
import soaprmi.RemoteException;
import soaprmi.util.Check;
import java.util.Map;
import java.util.HashMap;
import java.util.Iterator;
import java.lang.reflect.InvocationHandler;
import java.lang.reflect.Proxy;
import soaprmi.server.RemoteRef;

/**
 * This class represents the crux of our framework
 * implementation. It implements the Services interface
 * for the component developer and the ServicesPrivate
 * interface so the the proxy (ComponentID) in a remote
 * process can invoke methods from ConnectionServiceImpl.
 *
 * @version $Revision: 1.9 $  $Author: mgovinda $ $Date: 2001/05/08 23:43:06 $ (GMT) 
 * @author Madhusudhan Govindaraju [mailto:mgovinda@extreme.indiana.edu]
 */

public class ServicesImpl implements ServicesAndServicesConnection  {

  private Map providesPortList = new HashMap();
  private Map usesPortList = new HashMap();
  private CreationService creationService;
  private ConnectionService connectionService;
  private static Logger logger = Logger.getLogger();
  
  public ServicesImpl() throws XCATException {
    // init() ensures that the connection
    // and creation services are initialized
    init();
  }

  public PortInfo createPortInfo(String name, 
				 String type, 
				 String[] properties) {
    logger.warning("this method has not yet been implemented");
    // need to figure out how to use this method
    // FIXME FIXME
    return null;
  }
  
  public Port getPortNonblocking(String name) {
    logger.warning("this method has not yet been implemented");
    // FIXME FIXME
    // need to figure out how to use this
    return null;
  }

  public void unregisterUsesPort(String name) throws XCATException {
    logger.warning("this method has not yet been implemented");
    // FIXME FIXME
    // need to figure out how to use this
  }

  public void removeProvidesPort(String name) throws XCATException {
    logger.warning("this method has not yet been implemented");
    // FIXME FIXME
    // need to figure out how to use this

  }

  /**
   * @param nameOfPort that needs to be retrieved
   * Called by the users of components to get access to 
   * the usesPorts that have been registered.  Event the
   * providesPort is accessible using ths method, however
   * the component users are not expected to ever ask
   * for a provides port using this method.
   */
  public Port getPort(String nameOfPort) throws XCATException {
    // a runtime exception is thrown if the assert fails
    if (Check.ON) 
      Check.assert(nameOfPort != null);
    logger.finest("nameOfPort: " + nameOfPort);
    // the framework can provide pseudo components
    // like creation and connection as ports
    if (nameOfPort.equals("CreationService")) {
      return creationService;
    } else if (nameOfPort.equals("ConnectionService")) {
      return connectionService;
    }
    
    // try to retrieve the port from the list of registered uses port
    UsesPort usesPort = (UsesPort)getPortFromHashMap(nameOfPort, usesPortList);
    logger.finer("usesPort retrieved with fqn: " + usesPort.getClass());
    
    // if the port is a uses port then the return value from
    // the usesPortList will be not null. If the port has 
    // all void methods then getPort() need not block 
    // for a connect to have taken place. Or else the
    // getPort() method needs to block until this port
    // is connected.
    if (usesPort != null) {
      logger.finer("uses port with name " + nameOfPort + " retrieved");
      synchronized (usesPort) {
	if (!((usesPort.hasAllVoidMethods()).booleanValue()))
	  // block until there is a connection established with this port. When
	  // you are awaken, you already have a lock.
	  while (!((usesPort.isConnected()).booleanValue())) {
	    try {
	      logger.finer("ServerciesImpl.getPort: waiting for a connection\n");
	      usesPort.wait();
	      logger.finer("ServerciesImpl.getPort: usesPort is conencted. End of waiting period\n");
	    } catch (InterruptedException exp) {
	      String exceptionMessage = "ServerciesImpl.getPort: wait interrupted \n";
	      logger.severe(exceptionMessage);
	      throw new XCATException(exceptionMessage + exp.getMessage());
	    }
	  }
	usesPort.incrRefCount();
	return usesPort;
      } // end synchronized (usesPort) {
    } // end if (usesPort != null)
    
    // the requested port is not a uses port
    // search for it in the provides port
    ProvidesPort providesPortImpl = (ProvidesPort)
      getPortFromHashMap(nameOfPort, providesPortList);
    
    // if the port is not found, or a null reference for 
    // it is present, then throw an exception
    if (providesPortImpl == null) {
      String exceptionMessage =  "The following port has not been registered " +  nameOfPort;
      logger.severe("throwing exception: " + exceptionMessage);
      throw new XCATException(exceptionMessage);
    }
    return providesPortImpl;
  } // end getPort(...)
  
  
  /**
   * binds a provides port to a unique name for this Services object.
   * @param inPort is the provides port implementation provided by the user.
   * @param portInfo is the name and type of the provides port.
   * eg: myCore.addProvidesPort can take the following parameters;:
   * 'portReference' and 'new PortInfoImpl("input", "http://...#string")';
   * fqn  =  samples.idl.string.ProvidesString
   * The type has been defined as a wsdl type
   * http://extreme.indiana.edu/xcat/samples.wsdl#string
   */
  public void addProvidesPort(Port inPort, PortInfo portInfo) 
    throws XCATException {

    // The class ProvidesDyanmicSkelImpl is the 
    // remote object for all provides ports.
    InvocationHandler handler = null;
    try {
      handler = new ProvidesDynamicSkelImpl();
    } catch (RemoteException re) {
      String exceptionMessage = "unable to instantiate ProvidesDynamicSkelImpl ";
      logger.severe(exceptionMessage);
      throw new XCATException(exceptionMessage);
    }
    // use information in portInfo to fill in the
    // arguments of newProxyIntance from a global mapping
    // table. The entry in the table should have been 
    // added by the component user before making the
    // addProvidesPort(...) call.
    // It is very important that the dynamic proxy implement
    // the soaprmi.server.RemoteRef interface. SoapRMI knows that 
    // a particular object (in this case a proxy) needs to be 
    // serialized as  a port. Without this we cannot send provides port 
    // across components.
    // It is also important that the dyamic proxy implement 
    // the ProvidesPortImplInterface interface. This will
    // ensure that the method "assignProvidesPortImpl" method
    // can be invoked on the proxy. This method will bind
    // the user-provided reference to the providesPort with the
    // ProvidesDynamicSkelImpl

    String portName = portInfo.getName();
    String portType = portInfo.getType();
    Class[] arrayOfInterfaces = ProvidesPortMapping.getArrayOfInterfaces(portName, portType);

    for (int i=0; i<arrayOfInterfaces.length; i++) {
      logger.finest("dynamic interface " + arrayOfInterfaces[i].getName());
    }

    // create the dynamic proxy
    // if the type of the port is http://www...#string
    // then the component user is expected to add
    // the interface ProvidesString.class to the arrayOfInterfaces
    // and placed it in the mapping table, before the
    // invocation of addProvidesPort(...)
    ProvidesPort providesPortSkel = 
      (ProvidesPort) Proxy.newProxyInstance(
					    getClass().getClassLoader(),
					    arrayOfInterfaces,
					    handler
					    );
    if (providesPortSkel == null) {
      String exceptionMessage = ": usesPortStub is null! ";
      logger.severe(exceptionMessage);
      throw new XCATException(exceptionMessage);
    }
    logger.finer("registering usesPort with fqn : " + providesPortSkel.getClass());
    // assuming that the name of the port is unique.
    // it is therefore used as the key into the hashtable. 
    // It is certainlty desirable to use a combination of the 
    // name and type as the key to the hashtable. However, since
    // getPort(...) takes just the name of the port and not the type
    // it seems as if CCA forum expects the name to be unique.
    String keyForHashTable = portInfo.getName();
    PortRecord portRecord = new PortRecord(providesPortSkel, portInfo);
    providesPortList.put(keyForHashTable, portRecord);
    logger.finer(keyForHashTable + " added to the providesPortList");
    // this method is key storing the provides port given by the user
    // in the addProvidesPort(...) call as the implementation to be
    // used by the dynamic proxy for the provides port.
    // Since the dynamic proxy implements both ProvidesPort and
    // ProvidesPortImplInterface, we can cast it to ProvidesPortImplInterface
    // before invoking the assignProvidesPortImpl(...) method
    ((ProvidesPortImplInterface)providesPortSkel).assignProvidesPortImpl((ProvidesPort)inPort);
  }

  /**
   * Used by the user to add a uses port to the framework.
   * @param portInfo indicates the name and type of the port requested.
   * eg: myCore.registerUsesPort(new PortInfo("output", "http://...#string"));
   * fqn  =  xcat.framework.idl.String.UsesStringPortSkel
   */

  public void registerUsesPort(PortInfo portInfo)  
    throws XCATException {

    InvocationHandler handler = new UsesDynamicStubImpl();
    // use information in portInfo to fill in the
    // arguments of newProxyIntance from a global mapping
    // table. The global mapping table entry should have been
    // entered by the user before invoking registerUsesPort(...)
    String portName = portInfo.getName();
    String portType = portInfo.getType();
    // if the type of the port is http://www...#string
    // then the user should have added UsesString.class
    // to the list of interfaces
    Class[] arrayOfInterfaces = UsesPortMapping.getArrayOfInterfaces(portName, portType);
    
    // create a dynamic stub for a port of given type
    UsesPort usesPortStub = 
      (UsesPort) Proxy.newProxyInstance(
					getClass().getClassLoader(),
					arrayOfInterfaces,
					handler
					);
    
    if (usesPortStub == null) {
      String exceptionMessage = "registerUsesPort: usesPortStub is null! ";
      logger.severe(exceptionMessage);
      throw new XCATException(exceptionMessage);
    }
    logger.finer("registering usesPort with fqn : " + usesPortStub.getClass());
    // assuming that the name of the port is unique,
    // and thus using it as the key in the hashtable
    String keyForHashTable = portInfo.getName();
    PortRecord portRecord = new PortRecord(usesPortStub, portInfo);
    usesPortList.put(keyForHashTable, portRecord);
    logger.finer(keyForHashTable + " added to the usesPortList");
  }
  
  /**
   * @param nameOfPort to be unblocked
   * This method basically decrements the ref
   * count associated with the number of connections
   * in this port.
   */
  public void releasePort(String nameOfPort) throws XCATException {

    logger.finest("called to release port " + nameOfPort);
    UsesPort usesPort = (UsesPort) 
      getPortFromHashMap(nameOfPort, usesPortList);
    if (usesPort != null) {
      synchronized (usesPort) {	 
	usesPort.decrRefCount();
	logger.finest("decremented refCount for usesPort: " + nameOfPort);
      }
    }
  }

  /**
    *---------------------XCAT-INTERNAL-METHODS-------------------------*
    * The next set of the methods are internal methods used just by the
    * xcat framework. These methods are not to be used by a compoent
    * developer or user.
    *-------------------------------------------------------------------*
    */
  

  /**
   * xcatConnect is a method belonging to the ServicesInternal interface.
   * Called by the ComponentID in a remote process. This method is used in the
   * ConnectionServiceImplementation. It gets a uses port, adds the provides
   * port as the listener and then wakes up any threads waiting for a connection 
   * to be made.
   */

  public void xcatConnect(String usesPortName, ProvidesPort providesPort) 
    throws XCATException {
    logger.finest("usesPortName: " + usesPortName);
    synchronized(this) {
      UsesPort usesPort = (UsesPort) 
	getPortFromHashMap (usesPortName, usesPortList);
      if (usesPort != null) {
	logger.finer("retrieved usesPort for " + usesPortName);
	synchronized (usesPort) {
	  // if the uses port has all void methods
	  // then a connect can be done without checking
	  // whether or not the port has been released.
	  // If the port has non-void methods, then 
	  // all previous getPort() calls should have
	  // been followed by a release() call on the port.
	  if (((usesPort.hasAllVoidMethods()).booleanValue()) ||
	      (!((usesPort.hasAllVoidMethods()).booleanValue()) 
	       && (((usesPort.getRefCount()).intValue()) == 0))) {
	    usesPort.addListener((ProvidesPort)providesPort);
	    logger.finer(usesPortName + " added as a listener");
	    usesPort.notifyAll();
	  } else {
	    // an exception should be thrown as the connect cannot 
	    // be completed
	    String exceptionMessage = "ServerciesImpl.xcatConnect: uses port" +
	      usesPortName + " is locked, cannot connect\n";
	    logger.severe("throwing XCATException " + exceptionMessage);
	    throw new XCATException(exceptionMessage);
	  }
	}
      } else {  // if (userPort != null)
	String exceptionMessage = "ServerciesImpl.xcatConnect: failed to find uses port: " + usesPortName + " to connect to given providesPort\n";
	logger.severe("throwing XCATException " + exceptionMessage);
	throw new XCATException(exceptionMessage);
      }
    } // end synchronized(this)
  }
  
  /**
   * @param usesPortName the name of the usesPort that needs to be disconnected
   * @param providesPort the providesPort to which the usesPort is connected
   * Called by the ComponentID in a remote process.
   * This method is used in the ConnectionServiceImplementation.
   */
  public void xcatDisconnect(String usesPortName, ProvidesPort providesPort) 
    throws XCATException {
    logger.finest("usesPortName: " + usesPortName);
    synchronized(this) {
      try {
	UsesPort usesPort = (UsesPort)getPortFromHashMap(usesPortName, usesPortList);
	if (usesPort != null) {
	  logger.finer("retrieved usesPort for " + usesPortName);
	  synchronized (usesPort) {
	    if (((usesPort.hasAllVoidMethods()).booleanValue()) ||
		(!((usesPort.hasAllVoidMethods()).booleanValue())
		 && (((usesPort.getRefCount()).intValue()) == 0))) {
	      usesPort.removeListener((ProvidesPort)providesPort);
	      logger.finer(usesPortName + " has been disconnected");
	    } else {
	      String exceptionMessage = "ServerciesImpl.xcatDisconnect: uses port" +
		usesPortName + " is locked, cannot disconnect";
	      logger.severe("throwing XCATException " + exceptionMessage);
	      throw new XCATException(exceptionMessage);
	    }
	  }
	} else {
	  String exceptionMessage = "ServerciesImpl.xcatDisconnect: failed to find uses port: " + usesPortName;
	  logger.severe("throwing XCATException " + exceptionMessage);
	  throw new XCATException(exceptionMessage);
	}
      } catch (Exception e) {
	// FIXME 
	// should not catch an exception of type Exception
	throw new XCATException(e.getMessage());
      }
    }
  }

  /**
   *  Called by the ComponentID in a remote process.
   * This method is used in the ConnectionServiceImplementation.
   */
    public ProvidesPort xcatGetProvidesPort(String providesPortName) 
  // public Remote xcatGetProvidesPort(String providesPortName) 
    throws XCATException {
    logger.finest("called for providesPortName: " + providesPortName);
    synchronized(this) {
      ProvidesPort providesPort = (ProvidesPort)
	getPortFromHashMap(providesPortName, providesPortList);
      logger.finest("retrieved providesPort for " + providesPortName + ":" + providesPort.getClass());
      return providesPort;
    }
  }

  /**
   * Export the given port with the given portInfo. Note that this is not an
   * actual port implementation object but a proxy for some provides port
   * implementation in another context. However, the reference is assumed to
   * be a valid provides port reference.
   */
  public void xcatExportAs(PortInfo newPortInfo, ProvidesPort providesPort) 
    throws XCATException {
    logger.finest("called with portInfo " + newPortInfo.toString());
    String keyForHashMap = newPortInfo.getName();
    providesPortList.put(keyForHashMap, (ProvidesPort)providesPort);
  }

  /**
   * FIXME, need to figure out why this is required
   */
  public void xcatProvideTo(String usesPortName, ProvidesPort providesPort) 
       throws XCATException {
    logger.warning("this method is not supported as of now");
    throw new XCATException("the method xcatProvideTo is not supported");
    //return xcatConnect (usesPortName, providesPort);
  }

  /**
   * called from exportAs() method in ConnectionServiceImpl.java
   */
  public String xcatGetPortTypeForPortName(String portName) throws XCATException {
    logger.warning("this method is not supported as of now");
    throw new XCATException("the method xcatGetPortTypeForPortName is not supported");
  }

  /**
   * Once a component is instantiated it can get
   * a handle to its componentID by invoking this
   * method on its services object
   */
  public ComponentID getComponentID() throws XCATException {
    logger.finest("called");
    try {
      return new ComponentIDImpl(this);
    } catch (RemoteException re) {
      throw new XCATException(re.getMessage());
    }
  }

  /**
    *---------------------PRIVATE-METHODS-------------------------*
    * The rest of the methods are private methods used just by the
    * methods in this class as helper methods
    */
    
  /**
   * called from the constructor. Within
   * a JVM the same creation and connection service is used.
   * If this policy is changed then just the implementation
   * of this method needs to be chaneged.
   * FrameworkServicesSingletonFactory implements the
   * "singleton" pattern, as can be deduced from the name.
   */
  private void init() {
    logger.finest("called");
    creationService = 
      FrameworkServicesSingletonFactory.getCreationService();
    connectionService = 
      FrameworkServicesSingletonFactory.getConnectionService();
  }
  
  /**
   * @param nameOfPort that needs to be searched.
   * @return Port can be either usesPort or providesPort.
   * getPort() casts the return value accordingly.
   * This method is private as it is called locally
   * by getPort() and should not be called by any ther class.
   * This method does not throw an exception but instead returns
   * null if a port corresponding to the name is not found.
   * This is because, the null return value is detected by
   * getPort() and if need be, an appropriate exception is thrown.
   */
  private Port getPortFromHashMap(String nameOfPort, Map hashMap) 
    throws XCATException {
    logger.finest("called for nameOfPort: " + nameOfPort);
    Iterator iter = hashMap.entrySet().iterator();
    while (iter.hasNext()) {
      Map.Entry entry = (Map.Entry)iter.next();
      String keyName = (String)entry.getKey();
      if (keyName.equals(nameOfPort)) {
	PortRecord portRecord = (PortRecord)entry.getValue();
	logger.finest("found a port in list for " + nameOfPort);
	return portRecord.getPort();
      }
    }
    throw new XCATException("The following port is not found int the list of known ports: " + nameOfPort);
  }

} // end public class ServicesImpl implements Services, ComponentIDconnection {
