package xcat.framework.connection;

import soaprmi.RemoteException;
import xcat.framework.ccacore.Port;
import xcat.framework.ccacore.ProvidesPort;
import xcat.framework.ccacore.ComponentID;

/**
 * The user or the compositionEngine
 * uses this interface to tie up componet
 * instances together. The connection service
 * is a port that provides connect, disconnect
 * exportAs and provideTo service.
 *
 * @version $Revision: 1.5 $  $Author: mgovinda $ $Date: 2001/05/08 20:06:07 $ (GMT) 
 * @author Madhusudhan Govindaraju [mailto:mgovinda@extreme.indiana.edu]
 */
 
public interface ConnectionService extends Port {

  /**
   * @param idUser: ComponentID of the component with UsesPort
   * @param usesportName: the unique name of the usesPort
   * @param idProv: ComponentID of the component with providesPort
   * @param provportName: the unique name of the providesPort
   * this method is used to connect the uses and the provides port
   * of two components. The ConnectionService invokes internal 
   * methods on the componentID to obtain the provides port of 
   * idProv and then invokes another internal method to add this
   * providesPort to the list in the usesPort of idUser.
   */
  public void connect(ComponentID idUser, 
		      String usesportname, 
		      ComponentID idProv, 
		      String provportName) 
    throws RemoteException;

  /**
   * @param idUser: ComponentID of the component with UsesPort
   * @param usesportName: the unique name of the usesPort
   * @param idProv: ComponentID of the component with providesPort
   * @param provportName: the unique name of the providesPort
   * @param timeout: the number of seconds to wait before disconnecting
   * this method is just the opposite of the "connect" method.
   */
  public void disconnect(ComponentID idUser, 
			 String usesportname, 
			 ComponentID idProv, 
			 String provportname, double timeout) 
    throws RemoteException;

  /**
   * @param idMe: The componentID of the component that gets a proxy provides port
   * @param idSub: The componentID of the component that wants to export the provides port
   * @param portName: name of idSub's provides port
   * @param newPortName: name of idMe's proxy provides port
   * export idSub's provides port with name "portName" as idMe's provides port
   * with name "newPortName". Currently it's not implemented for uses
   * port.
   */
  public void exportAs(ComponentID idMe, 
		       ComponentID idSub, 
		       String portname, 
		       String newportname) 
    throws RemoteException;
  
  /**
   * Provide the provides port "providesPort" to the componentID idSub's uses
   * port named "usesPortName".
   */
  public void provideTo(ComponentID idSub, 
			String usesPortName, 
			ProvidesPort providesPort) 
    throws RemoteException;
}
                



