package xcat.framework.connection;

import xcat.framework.ccacore.ComponentID;
import xcat.framework.ccacore.ComponentIDConnection;
import xcat.framework.ccacore.Port;
import xcat.framework.ccacore.ProvidesPort;
import xcat.framework.ccacore.UsesPort;
import xcat.framework.ccacore.PortInfoImpl;
import xcat.framework.ccacore.PortInfo;
import soaprmi.RemoteException;
import soaprmi.Remote;
import soaprmi.soaprpc.SoapServices;
import soaprmi.util.Logger;
import soaprmi.util.Level;

/**
 *  ConnectionServiceImpl implements the ConnectionServices
 *  interface. It invokes methods from the ComponentIDprivate
 *  interface to achieve its functionality. The User is not
 *  exposed to the implementation details of the framework or
 *  the ConnectionService.
 * @version $Revision: 1.8 $  $Author: mgovinda $ $Date: 2001/05/08 23:43:07 $ (GMT) 
 * @author Madhusudhan Govindaraju [mailto:mgovinda@extreme.indiana.edu]
 */

public class ConnectionServiceImpl implements ConnectionService {

  private static Logger logger = Logger.getLogger();

  public void connect(ComponentID idUses, 
		      String usesPortName, 
		      ComponentID idProv, 
		      String provPortName) 
    throws RemoteException {
    logger.finest("connecting usesPort: " + usesPortName + " and providesPort " + provPortName);
    logger.finer("attempting xcatGetProvidesPort");
    ProvidesPort providesPort = ((ComponentIDConnection)idProv).xcatGetProvidesPort(provPortName);
    logger.finer("xcatGetProvidesPort done");
    String xmlRepOfRef = SoapServices.getDefault().getStartpointXml((Remote)providesPort);
    logger.finer("xcatGetProvidesPort returned : " + xmlRepOfRef);
    logger.finer("attempting xcatConnect");
    ((ComponentIDConnection)idUses).xcatConnect(usesPortName, providesPort);
    logger.finer("xcatConnect done");
  }

  public void disconnect(ComponentID idUses, 
			 String usesPortName, 
			 ComponentID idProv, 
			 String provPortName, 
			 double timeout) 
    throws RemoteException {

    logger.finest("disconnecting usesPort: " + usesPortName + " and providesPort " + provPortName);

    // need to wait for timeout milliseconds before attempting a disconnect
    // FIXME FIXME

    ProvidesPort providesPort = ((ComponentIDConnection)idProv).xcatGetProvidesPort(provPortName);
    logger.finer("attempting xcatDisconnect");
    ((ComponentIDConnection)idUses).xcatDisconnect(usesPortName, providesPort);
    logger.finer("xcatDisconnect done");
  }
    

  /**
   * export idSub's provides port with name "portName" as idMe's provides port
   * with name "newPortName". Currently it's not implemented for uses
   * port. FIXME FIXME
   */
  public void exportAs(ComponentID idMe, 
		       ComponentID idSub, 
		       String portName, 
		       String newPortName) 
    throws RemoteException {

    logger.finest("exportAs Port:" + portName + " as Port:" + newPortName);
    ProvidesPort providesPort = null;
    String portType = null;
    providesPort =
      ((ComponentIDConnection)idSub).xcatGetProvidesPort(portName);
    
    // get the portType for this port
    portType = 
      ((ComponentIDConnection)idSub).xcatGetPortTypeForPortName(portName);

    logger.finer("exportAs: the portType found is: " + portType);
    // get the port info for this port
    PortInfoImpl newPortInfo = new PortInfoImpl(newPortName, portType);
    ((ComponentIDConnection)idMe).xcatExportAs (newPortInfo, providesPort);
    logger.finer("done");
  }

  /**
   * Provide the provides port "providesPort" to the componentID idSub's uses
   * port named "usesPortName".
   */
  public void provideTo(ComponentID idSub, 
			String usesPortName, 
			ProvidesPort providesPort) 
    throws RemoteException {

    logger.finest("XCATServicesImpl:provideTo usesPortName:" + usesPortName);
    ((ComponentIDConnection)idSub).xcatProvideTo(usesPortName, providesPort);
    logger.finer("done");
  }
}




