package xcat.framework.util;

import xcat.framework.ccacore.ComponentID;
import soaprmi.RemoteException;
import soaprmi.soaprpc.SoapServices;
import soaprmi.util.Logger;
import soaprmi.util.Level;

/**
 * @version $Revision: 1.3 $  $Author: mgovinda $ $Date: 2001/05/08 20:07:05 $ (GMT) 
 * @author Madhusudhan Govindaraju [mailto:mgovinda@extreme.indiana.edu]
 **/

/**
 * The SoapConnectionClient connects to the SoapConnectionServer
 * and passes on the componentID of the instantiated component.
 */
public class SoapConnectionClient {

  // Since no registry is being used, both the SoapConnectionServer
  // and SoapConnectionClient use the same name of the "service"
  // to communicate.
  private final static String SERVICE_NAME = SoapConnectionServerImpl.SERVICE_NAME;
  private static Logger logger = Logger.getLogger();
  
  /**
   * @param creationServiceHost: host at which SoapConnectionServer is waiting
   * @param creationServicePort: port at which SoapConnectionServer is waiting
   * @param componentID: componentID of the instantiated component.
   */
  public static void sendMyComponentID(String creationServiceHost, 
				       int creationServicePort, 
				       ComponentID componentID) 
    throws RemoteException {
    logger.finest("called");
    String serverLoc = "http://" + creationServiceHost + ":" + creationServicePort +"/"+ SERVICE_NAME; 
    logger.finer("serverLoc is at: " + serverLoc);
    SoapConnectionServer remoteRef = getServerRef(serverLoc);
    logger.finer("attempting to send componentID to " + remoteRef);
    remoteRef.sendComponentID(componentID);
  }

  /**
   *@param serverLoc: the url at which the SoapConnectionServer
   * is waiting for the componentID
   */
  private static SoapConnectionServer getServerRef(String serverLoc)
    throws RemoteException {
    
    return (SoapConnectionServer) SoapServices.getDefault().createStartpoint
      (serverLoc,
       new Class[]{SoapConnectionServer.class},
       SERVICE_NAME      
       );
  }
}

