
/**
 * @version $Revision: 1.1 $ $Author: mgovinda $ $Date: 2001/04/24 18:50:08 $ (GMT)
**/

package xcat.framework.util;

import java.net.URL;
import java.net.MalformedURLException;
import java.net.URLConnection;
import java.util.Hashtable;
import java.io.IOException;
import java.io.InputStream;

public class XCATClassLoader {
  /*
   * Disallow anyone from creating one of these.
   */
  private static Hashtable cloaders = new Hashtable();
  
  private XCATClassLoader() {}
  
  public static Class loadClass(String name)
    throws MalformedURLException, ClassNotFoundException {
    String codebasestring = System.getProperty("java.rmi.server.codebase");
    if(codebasestring == null)
      return loadClass(null, name);
    else
      return loadClass(new URL(codebasestring), name);
  }
  
  public static Class loadClass(URL codebase, String name)
    throws MalformedURLException, ClassNotFoundException {
    if(codebase == null)
      return Class.forName(name);
    
    URLClassLoader ucl = (URLClassLoader)cloaders.get(codebase);
    if(ucl == null) {
      URL[] urls = new URL[1];
      urls[0] = codebase;
      ucl = new URLClassLoader(urls);
      cloaders.put(codebase, ucl);
    }
    return ucl.loadClass(name);
  }
  
  public static Object getSecurityContext(ClassLoader loader) {
    return null;
  }
}

/** 
 * The following class is a systemclass since 1.2 so it can be commented
 * out when working with 1.2 (it will still work, even if it is not
 * commented out though.
 */
class URLClassLoader extends ClassLoader {
  URL[] urls = null;
  
  public URLClassLoader(URL[] urls) {
    this.urls = urls;
  }
  
  public Class loadClass(String name, boolean resolve)
    throws ClassNotFoundException {
    Class c = findLoadedClass(name);
    if(c == null) {
      try {
	c = Class.forName(name);
      }
      catch(ClassNotFoundException cnfe) {
	int i = 0;
	String classname = name.replace('.', '/') + ".class";
	c = null;
	while((c == null) && (i < urls.length)) {
	  URL classurl = null;
	  try {
	    classurl = new URL(urls[i], classname);
	  }
	  catch(MalformedURLException mue) {
	    classurl = null;
	  }
	  if(classurl != null) {
	    byte[] bytes = null;
	    try {
	      int pos, len, avail;
	      //Using a URLConnection does not seem to work...
	      // URLConnection ucon = classurl.openConnection();
	      // System.out.println("Content type: " + ucon.getContentType());
	      // System.out.println("length      : " + ucon.getContentLength());
	      // System.out.println("Date        : " + ucon.getDate());
	      // Object urlobj = ucon.getContent();
	      // System.out.println("Content = " + urlobj.getClass());
	      //  bytes = (byte[])urlobj;
	      
	      //So I use a stream...
	      InputStream is = classurl.openStream();
	      // This may not always work !!!!!!
	      avail = is.available();                     
	      pos = 0;
	      len = 0;
	      bytes = new byte[avail];
	      while((len += is.read(bytes, pos, avail - len)) >= 0)
		pos = len;
	      is.close();
	    }
	    catch(IOException ioe) {
	      System.err.println("URLClassLoader: error while reading from " + classurl);
	      ioe.printStackTrace();
	      bytes = null;
	    }
	    if(bytes != null) {
	      try {
		c = defineClass(null, bytes, 0, bytes.length);
		if(resolve)
		  resolveClass(c);
	      }
	      catch(ClassFormatError cfe) {
		System.err.println("Could not define class from " + classurl + "(" + bytes.length + " bytes)");
		c = null;
	      }
	    }
	  }
	  i++;
	}
	if(c == null)
	  throw new ClassNotFoundException(name);
      }
      //System.out.println("Class loaded");
    }
    return c;
  }
}
